import 'dart:core';

import 'package:fis_jsonrpc/client_base.dart';
import 'package:fis_common/json_convert.dart';

import 'device.m.dart';

class DeviceService extends JsonRpcClientBase {
	DeviceService(
		String host, {
		String serviceName = "IDeviceService",
		Map<String, String>? headers,
		int? timeout,
	}) : super(
						host,
						serviceName,
						headers: headers,
						timeout: timeout,
				) {
		/// 注册响应实体反序列化处理器
		FJsonConvert.setDecoder((map) => DeviceInfoDTO.fromJson(map));
		FJsonConvert.setDecoder((map) => PageCollection<DeviceInfoDTO>.fromJson(map));
	}

	Future<bool> createDeviceInfoAsync(CreateDeviceRequest request) async {
		var rpcRst = await call("CreateDeviceInfoAsync", request);
		return rpcRst;
	}

	Future<DeviceInfoDTO> getDeviceInfoAsync(String deviceCode) async {
		var rpcRst = await call("GetDeviceInfoAsync", deviceCode);
		var result = DeviceInfoDTO.fromJson(rpcRst as Map<String, dynamic>);
		return result;
	}

	Future<PageCollection<DeviceInfoDTO>> getDeviceInfoPageAsync(PageRequest request) async {
		var rpcRst = await call("GetDeviceInfoPageAsync", request);
		var result = PageCollection<DeviceInfoDTO>.fromJson(rpcRst as Map<String, dynamic>);
		return result;
	}

	Future<bool> createShareDeviceToUserAsync(CreateShareDeviceToUserRequest request) async {
		var rpcRst = await call("CreateShareDeviceToUserAsync", request);
		return rpcRst;
	}

	Future<bool> deleteShareDeviceToUserAsync(DeleteShareDeviceToUserRequest request) async {
		var rpcRst = await call("DeleteShareDeviceToUserAsync", request);
		return rpcRst;
	}

}

